@@ -14,23 +14,16 @@
 
  ------------------------------------------------------------- */
 
-#include <qcolor.h>
-#include <qregexp.h>
 #include <qclipboard.h>
-#include <qsplitter.h>
-#include <qpushbutton.h>
 
-#include <kaction.h>
+#include <kstdaccel.h>
 #include <kstdaction.h>
-#include <kapp.h>
+#include <kapplication.h>
 #include <kstatusbar.h>
 #include <klocale.h>
-#include <kconfig.h>
 #include <kwin.h>
-#include <kkeydialog.h>
 #include <kedittoolbar.h>
 #include <kdebug.h>
-#include <kcombobox.h>
 #include <dcopclient.h>
 
 #include "actions.h"
@@ -57,17 +50,16 @@ DictInterface *interface;
 GlobalData *global;
 
 
-TopLevel::TopLevel()
-: KMainWindow(), DCOPObject("KDictIface"), optDlg(0L), setsDlg(0L), stopRef(0)
+TopLevel::TopLevel(QWidget* parent, const char* name)
+  : DCOPObject("KDictIface"), KMainWindow(parent, name, WType_TopLevel),
+    optDlg(0L), setsDlg(0L), stopRef(0)
 {
   kapp->dcopClient()->setDefaultObject(objId());
   kapp->setMainWidget(this);
-  setCaption(i18n("The KDE Dict(ionary) Client"));
 
   global = new GlobalData();
   global->topLevel = this;
   global->read();
-
   interface = new DictInterface();
   connect(interface,SIGNAL(infoReady()),SLOT(stratDbChanged()));
   connect(interface,SIGNAL(started(const QString&)),SLOT(clientStarted(const QString&)));
@@ -88,7 +80,6 @@ TopLevel::TopLevel()
   connect(matchView,SIGNAL(matchRequested(const QString&)),SLOT(match(const QString&)));
   connect(matchView,SIGNAL(clipboardRequested()),SLOT(matchClipboard()));
   connect(matchView,SIGNAL(windowClosed()),SLOT(toggleMatchListShow()));
-
   connect(&resetStatusbarTimer,SIGNAL(timeout()),SLOT(resetStatusbar()));
 
   setupStatusBar();
@@ -96,21 +87,18 @@ TopLevel::TopLevel()
   recreateGUI();
   buildHistMenu();
 
-  if (global->showMatchList) {    // show splitter, html view & match list
-    if (global->swallowMatchList) {
-      splitter = new QSplitter(QSplitter::Horizontal,this);
-      splitter->setOpaqueResize(true);
-      queryView->reparent(splitter,0,queryView->pos(),true);
-      matchView->reparent(splitter,0,matchView->pos(),true);
-      setCentralWidget(splitter);
-      splitter->setResizeMode(matchView,QSplitter::KeepSize);
-      adjustMatchViewSize();
-    } else {
-      setCentralWidget(queryView);
-      matchView->show();
-      adjustMatchViewSize();
-    }
-  } else {                       // show only html view
+  if (global->showMatchList)
+  {    // show splitter, html view & match list
+    splitter = new QSplitter(QSplitter::Horizontal,this);
+    splitter->setOpaqueResize( KGlobalSettings::opaqueResize() );
+    queryView->reparent(splitter,0,queryView->pos(),true);
+    matchView->reparent(splitter,0,matchView->pos(),true);
+    setCentralWidget(splitter);
+    splitter->setResizeMode(matchView,QSplitter::KeepSize);
+    adjustMatchViewSize();
+  }
+  else
+  {                       // show only html view
       setCentralWidget(queryView);
       matchView->hide();
   }
@@ -118,9 +106,6 @@ TopLevel::TopLevel()
   //apply settings
   resize(600,390);
   applyMainWindowSettings(KGlobal::config(),"toplevel_options");
-  actToggleToolbar->setChecked(!toolBar()->isHidden());
-  actToggleStatusbar->setChecked(!statusBar()->isHidden());
-
   stratDbChanged();              // fill combos, build menus
 
   actQueryCombo->setFocus();        // place cursor in combobox
@@ -150,8 +135,7 @@ void TopLevel::quit()
 void TopLevel::makeActiveWindow()
 {
   kdDebug(5004) << "*DCOP call* TopLevel::makeActiveWindow()" << endl;
-  KWin::setOnDesktop(winId(),KWin::currentDesktop());
-  KWin::setActiveWindow(winId());
+  raiseWindow();
 }
 
 
@@ -159,8 +143,7 @@ void TopLevel::definePhrase(QString phrase)
 {
   kdDebug(5004) << "*DCOP call* TopLevel::definePhrase()" << endl;
   define(phrase);
-  KWin::setOnDesktop(winId(),KWin::currentDesktop());
-  KWin::setActiveWindow(winId());
+  raiseWindow();
 }
 
 
@@ -168,8 +151,7 @@ void TopLevel::matchPhrase(QString phrase)
 {
   kdDebug(5004) << "*DCOP call* TopLevel::matchPhrase()" << endl;
   match(phrase);
-  KWin::setOnDesktop(winId(),KWin::currentDesktop());
-  KWin::setActiveWindow(winId());
+  raiseWindow();
 }
 
 
@@ -177,8 +159,7 @@ void TopLevel::defineClipboardContent()
 {
   kdDebug(5004) << "*DCOP call* TopLevel::defineClipboardContent()" << endl;
   defineClipboard();
-  KWin::setOnDesktop(winId(),KWin::currentDesktop());
-  KWin::setActiveWindow(winId());
+  raiseWindow();
 }
 
 
@@ -186,8 +167,7 @@ void TopLevel::matchClipboardContent()
 {
   kdDebug(5004) << "*DCOP call* TopLevel::matchClipboardContent()" << endl;
   matchClipboard();
-  KWin::setOnDesktop(winId(),KWin::currentDesktop());
-  KWin::setActiveWindow(winId());
+  raiseWindow();
 }
 
 
@@ -280,7 +260,13 @@ void TopLevel::define(const QString &query)
 void TopLevel::defineClipboard()
 {
   kdDebug(5004) << "TopLevel::defineClipboard()" << endl;
-  define(kapp->clipboard()->text());
+  kapp->clipboard()->setSelectionMode(true);
+  QString text = kapp->clipboard()->text();
+  if (text.isEmpty()) {
+    kapp->clipboard()->setSelectionMode(false);
+    text = kapp->clipboard()->text();
+  }
+  define(text);
 }
 
 
@@ -295,7 +281,13 @@ void TopLevel::match(const QString &query)
 void TopLevel::matchClipboard()
 {
   kdDebug(5004) << "TopLevel::matchClipboard()" << endl;
-  match(kapp->clipboard()->text());
+  kapp->clipboard()->setSelectionMode(true);
+  QString text = kapp->clipboard()->text();
+  if (text.isEmpty()) {
+    kapp->clipboard()->setSelectionMode(false);
+    text = kapp->clipboard()->text();
+  }
+  match(text);
 }
 
 
@@ -304,8 +296,9 @@ bool TopLevel::queryClose()
   kdDebug(5004) << "TopLevel::queryClose()" << endl;
 
   saveMainWindowSettings(KGlobal::config(),"toplevel_options");
-
   saveMatchViewSize();
+  global->queryComboCompletionMode = actQueryCombo->completionMode();
+
   global->write();
 
   return true;
@@ -316,11 +309,11 @@ void TopLevel::setupActions()
 {
   // file menu...
   actSave = KStdAction::save(queryView, SLOT(saveQuery()), actionCollection());
-  actSave->setText("&Save...");
+  actSave->setText(i18n("&Save As..."));
   actSave->setEnabled(false);
   actPrint = KStdAction::print(queryView, SLOT(printQuery()), actionCollection());
   actPrint->setEnabled(false);
-  actStartQuery = new KAction(i18n("St&art Query"),"redo", 0 , this,
+  actStartQuery = new KAction(i18n("St&art Query"),"reload", 0 , this,
                   SLOT(doDefine()), actionCollection(), "start_query");
   actStopQuery = new KAction(i18n("St&op Query"),"stop", 0 , this,
                  SLOT(stopClients()), actionCollection(), "stop_query");
@@ -338,12 +331,12 @@ void TopLevel::setupActions()
   KStdAction::find(queryView, SLOT(showFindDialog()), actionCollection());
 
   // history menu...
-  actBack = new KToolBarPopupAction(i18n("&Back"), "back", KStdAccel::key(KStdAccel::Back),
+  actBack = new KToolBarPopupAction(i18n("&Back"), "back", KStdAccel::shortcut(KStdAccel::Back),
                                     queryView, SLOT(browseBack()), actionCollection(),"browse_back");
   actBack->setDelayed(true);
   actBack->setStickyMenu(false);
   actBack->setEnabled(false);
-  actForward = new KToolBarPopupAction(i18n("&Forward"), "forward", KStdAccel::key(KStdAccel::Forward),
+  actForward = new KToolBarPopupAction(i18n("&Forward"), "forward", KStdAccel::shortcut(KStdAccel::Forward),
                                        queryView, SLOT(browseForward()), actionCollection(),"browse_forward");
   actForward->setDelayed(true);
   actForward->setStickyMenu(false);
@@ -364,15 +357,15 @@ void TopLevel::setupActions()
               SLOT(showInfo()), actionCollection(), "server_info");
 
   // settings menu...
-  actToggleToolbar = KStdAction::showToolbar(this, SLOT(toggleToolbar()), actionCollection());
-  actToggleStatusbar = KStdAction::showStatusbar(this, SLOT(toggleStatusbar()), actionCollection());
+  createStandardStatusBarAction();
+  setStandardToolBarMenuEnabled(true);
+
   actShowMatchList = new KToggleAction(i18n("Show &Match List"), 0 , this,
                                  SLOT(toggleMatchListShow()), actionCollection(), "show_match");
-  actSwallowMatchList = new KToggleAction(i18n("Swallow Match &List"), 0 , this,
-                                    SLOT(toggleMatchListSwallow()), actionCollection(), "swallow_match");
+  actShowMatchList->setCheckedState(i18n("Hide &Match List"));
   actShowMatchList->setChecked(global->showMatchList);
-  actSwallowMatchList->setChecked(global->swallowMatchList);
-  KStdAction::keyBindings(this, SLOT(slotConfKeys()), actionCollection());
+  KStdAction::keyBindings(guiFactory(), SLOT(configureShortcuts()),
+actionCollection());
   KStdAction::configureToolbars(this, SLOT(slotConfToolbar()), actionCollection());
   KStdAction::preferences(this, SLOT(showOptionsDialog()), actionCollection());
 
@@ -380,9 +373,10 @@ void TopLevel::setupActions()
   new KAction(i18n("Clear Input Field"), "query_erase", 0 , this,
               SLOT(clearInput()), actionCollection(), "clear_query");
 
-  actQueryLabel = new DictLabelAction(i18n("&Look for"), actionCollection(), "look_label");
+  actQueryLabel = new DictLabelAction(i18n("&Look for:"), actionCollection(), "look_label");
   actQueryCombo = new DictComboAction(i18n("Query"), actionCollection(), "query_combo",true,true);
   connect(actQueryCombo,SIGNAL(activated(const QString &)), SLOT(define(const QString&)));
+  actQueryCombo->setCompletionMode(global->queryComboCompletionMode);
   actDbLabel = new DictLabelAction(i18n("&in"), actionCollection(), "in_label");
   actDbCombo = new DictComboAction(i18n("Databases"), actionCollection(), "db_combo",false,false);
   connect(actDbCombo,SIGNAL(activated(int)),SLOT(databaseSelected(int)));
@@ -414,8 +408,7 @@ void TopLevel::setupStatusBar()
 void TopLevel::recreateGUI()
 {
   kdDebug(5004) << "TopLevel::recreateGUI()" << endl;
-  createGUI("kdictui.rc");
-
+  createGUI("kdictui.rc", false);
   actQueryCombo->setList(global->queryHistory);
   actQueryCombo->clearEdit();
   actQueryLabel->setBuddy(actQueryCombo->widget());
@@ -423,14 +416,13 @@ void TopLevel::recreateGUI()
   actDbCombo->setList(global->databases);
   actDbCombo->setCurrentItem(global->currentDatabase);
   actDbLabel->setBuddy(actDbCombo->widget());
-
   int bwidth;
   if (actDefineBtn->widthHint() > actMatchBtn->widthHint())
     bwidth = actDefineBtn->widthHint();
   else
     bwidth = actMatchBtn->widthHint();
-  actDefineBtn->setWidth(bwidth-15);
-  actMatchBtn->setWidth(bwidth-15);
+  actDefineBtn->setWidth(bwidth);
+  actMatchBtn->setWidth(bwidth);
 }
 
 
@@ -438,7 +430,7 @@ void TopLevel::recreateGUI()
 void TopLevel::addCurrentInputToHistory()
 {
   QString text(actQueryCombo->currentText());
-  
+
   // maintain queryHistory
   global->queryHistory.remove(text);                     // no double entrys
   global->queryHistory.prepend(text);                   // prepend new item
@@ -465,9 +457,11 @@ void TopLevel::doDefine()
 {
   QString text(actQueryCombo->currentText());
 
-  if (!text.isEmpty()) {
+  if (!text.isEmpty())
+  {
     addCurrentInputToHistory();
-    interface->define(text.utf8());
+    actQueryCombo->selectAll();
+    interface->define(text);
   }
 }
 
@@ -476,14 +470,16 @@ void TopLevel::doMatch()
 {
   QString text(actQueryCombo->currentText());
 
-  if (!text.isEmpty()) {
+  if (!text.isEmpty())
+  {
     addCurrentInputToHistory();
+    actQueryCombo->selectAll();
 
     if (!global->showMatchList)
+    {
       toggleMatchListShow();
-    else
-      if (!global->swallowMatchList)
-        KWin::setActiveWindow(matchView->winId());
+    }
+
     matchView->match(text);
     setCaption(getShortString(text.simplifyWhiteSpace(),70));
   }
@@ -508,7 +504,7 @@ void TopLevel::buildHistMenu()
   unsigned int i = 0;
   while ((i<10)&&(i<global->queryHistory.count())) {
     historyActionList.append( new KAction(getShortString(global->queryHistory[i],70), 0, this, SLOT(queryHistMenu()),
-                                          0, global->queryHistory[i].utf8().data()) );
+                                          (QObject*)0, global->queryHistory[i].utf8().data()) );
     i++;
   }
 
@@ -547,7 +543,7 @@ void TopLevel::stratDbChanged()
 
   for (unsigned int i=0;i<global->serverDatabases.count();i++)
     dbActionList.append( new KAction(global->serverDatabases[i], 0, this, SLOT(dbInfoMenuClicked()),
-                                     0, global->serverDatabases[i].utf8().data()) );
+                                     (QObject*)0, global->serverDatabases[i].utf8().data()) );
 
   plugActionList("db_detail", dbActionList);
 }
@@ -630,132 +626,68 @@ void TopLevel::newCaption(const QString &s)
   setCaption(s);
 }
 
-
-void TopLevel::toggleStatusbar()
-{
-  if (statusBar()->isVisible())
-    statusBar()->hide();
-  else
-    statusBar()->show();
-}
-
-
-void TopLevel::toggleToolbar()
-{
-  if(toolBar()->isVisible())
-    toolBar()->hide();
-  else
-    toolBar()->show();
-}
-
-
 void TopLevel::toggleMatchListShow()
 {
   saveMatchViewSize();
-  if (global->showMatchList) {   // list is visible -> hide it
+  if (global->showMatchList) // list is visible -> hide it
+  {
     global->showMatchList = false;
-    if (global->swallowMatchList) {
-      queryView->reparent(this,0,queryView->pos(),true);
-      matchView->reparent(this,0,matchView->pos(),true);
-      matchView->hide();
-      delete splitter;
-      setCentralWidget(queryView);
-    } else {
-      matchView->hide();
-    }
-  } else {                      // list is not visible -> show it
+    queryView->reparent(this,0,queryView->pos(),true);
+    matchView->reparent(this,0,matchView->pos(),true);
+    matchView->hide();
+    delete splitter;
+    setCentralWidget(queryView);
+  }
+  else                       // list is not visible -> show it
+  {
     global->showMatchList = true;
-    if (global->swallowMatchList) {
-      splitter = new QSplitter(QSplitter::Horizontal,this);
-      splitter->setOpaqueResize(true);
-      setCentralWidget(splitter);
-      splitter->show();
-      queryView->reparent(splitter,0,queryView->pos(),true);
-      matchView->reparent(splitter,0,matchView->pos(),true);
-      splitter->setResizeMode(matchView,QSplitter::KeepSize);
-      adjustMatchViewSize();
-    } else {
-      matchView->reparent(0,0,matchView->pos(),true);
-      matchView->show();
-      adjustMatchViewSize();
-    }
+    splitter = new QSplitter(QSplitter::Horizontal,this);
+    splitter->setOpaqueResize( KGlobalSettings::opaqueResize() );
+    setCentralWidget(splitter);
+    splitter->show();
+    queryView->reparent(splitter,0,queryView->pos(),true);
+    matchView->reparent(splitter,0,matchView->pos(),true);
+    splitter->setResizeMode(matchView,QSplitter::KeepSize);
+    adjustMatchViewSize();
   }
+
   actShowMatchList->setChecked(global->showMatchList);
 }
 
 
-void TopLevel::toggleMatchListSwallow()
-{
-  saveMatchViewSize();
-  if (global->swallowMatchList) {    // list is swallowed -> unswallow
-    global->swallowMatchList = false;
-    if (global->showMatchList) {
-      queryView->reparent(this,0,queryView->pos(),true);
-      matchView->reparent(0,0,matchView->pos(),true);
-      matchView->show();
-      delete splitter;
-      setCentralWidget(queryView);
-      adjustMatchViewSize();
-    }
-  } else {                         // list is not swallowed -> swallow
-    global->swallowMatchList = true;
-    if (global->showMatchList) {
-      splitter = new QSplitter(QSplitter::Horizontal,this);
-      splitter->setOpaqueResize(true);
-      setCentralWidget(splitter);
-      splitter->show();
-      queryView->reparent(splitter,0,queryView->pos(),true);
-      matchView->reparent(splitter,0,matchView->pos(),true);
-      splitter->setResizeMode(matchView,QSplitter::KeepSize);
-      adjustMatchViewSize();
-    }
-  }
-}
-
-
 void TopLevel::saveMatchViewSize()
 {
-  if (global->showMatchList) {
-    if (global->swallowMatchList)
+  if (global->showMatchList)
+  {
       global->splitterSizes = splitter->sizes();
-    else
-      global->matchSize = matchView->size();
   }
 }
 
 
 void TopLevel::adjustMatchViewSize()
 {
-  if (global->swallowMatchList) {
-    if (global->splitterSizes.count()==2)
-      splitter->setSizes(global->splitterSizes);
-  } else {
-    if (global->matchSize.isValid())
-      matchView->resize(global->matchSize);
+  if (global->splitterSizes.count()==2)
+  {
+    splitter->setSizes(global->splitterSizes);
   }
-
-}
-
-
-void TopLevel::slotConfKeys()
-{
-  KKeyDialog::configureKeys(actionCollection(), xmlFile(), true, this);
 }
 
 
 void TopLevel::slotConfToolbar()
 {
   saveMainWindowSettings(KGlobal::config(),"toplevel_options");
-  KEditToolbar *dlg = new KEditToolbar(actionCollection(), "kdictui.rc");
+  KEditToolbar dlg(actionCollection(), "kdictui.rc");
+  connect(&dlg,SIGNAL( newToolbarConfig() ), this, SLOT( slotNewToolbarConfig() ));
+  dlg.exec();
+}
 
-  if (dlg->exec()) {
-    recreateGUI();
-    applyMainWindowSettings(KGlobal::config(),"toplevel_options");
-    buildHistMenu();   // actionlists must be inserted
-    stratDbChanged();
-  }
 
-  delete dlg;
+void TopLevel::slotNewToolbarConfig()
+{
+  recreateGUI();
+  applyMainWindowSettings(KGlobal::config(),"toplevel_options");
+  buildHistMenu();   // actionlists must be inserted
+  stratDbChanged();
 }
 
 
@@ -767,7 +699,7 @@ void TopLevel::showSetsDialog()
     connect(setsDlg,SIGNAL(dialogClosed()),this,SLOT(hideSetsDialog()));
     setsDlg->show();
   } else {
-    KWin::setActiveWindow(setsDlg->winId());
+    KWin::activateWindow(setsDlg->winId());
   }
 }
 
@@ -775,9 +707,7 @@ void TopLevel::showSetsDialog()
 void TopLevel::hideSetsDialog()
 {
   if (setsDlg) {
-    disconnect(setsDlg,SIGNAL(setsChanged()),this,SLOT(setsChanged()));
-    disconnect(setsDlg,SIGNAL(dialogClosed()),this,SLOT(hideSetsDialog()));
-    delete setsDlg;
+    setsDlg->delayedDestruct();
     setsDlg = 0L;
   }
 }
@@ -798,7 +728,7 @@ void TopLevel::showOptionsDialog()
     connect(optDlg,SIGNAL(finished()),this,SLOT(hideOptionsDialog()));
     optDlg->show();
   } else {
-    KWin::setActiveWindow(optDlg->winId());
+    KWin::activateWindow(optDlg->winId());
   }
 }
 
@@ -827,6 +757,21 @@ void TopLevel::optionsChanged()
   queryView->optionsChanged();              // inform html-view
 }
 
+void TopLevel::raiseWindow()
+{
+  // Bypass focus stealing prevention
+  kapp->updateUserTimestamp();
+
+  KWin::WindowInfo info = KWin::windowInfo( winId() );
+
+  if (  !info.isOnCurrentDesktop() )
+  {
+    KWin::setOnDesktop( winId(), KWin::currentDesktop() );
+  }
+
+  KWin::activateWindow(winId());
+}
+
 
 //--------------------------------
 
