@@ -21,6 +21,8 @@
 #include <qfile.h>
 #include <qtable.h>
 
+#include <kmessagebox.h>
+
 #ifndef WITHOUT_ARTS
 #include <arts/artsflow.h>
 #include <arts/connect.h>
@@ -31,6 +33,7 @@
 #include <iostream>
 #include <string>
 #include <klocale.h>
+#include <kstandarddirs.h>
 #include <kprocio.h>
 #include <kdebug.h>
 #include <qstring.h>
@@ -512,224 +515,101 @@ bool Interface_wireless_wirelessextensions::poll_device_info ()
   return true;
 }
 
-QString Interface_wireless_wirelessextensions::print_scanning_token (struct iw_event * event)
-{
-  QString
-    result;
-  /* Now, let's decode the event */
-  switch (event->cmd)
-    {
-    case SIOCGIWESSID:
-	char
-	  essid[IW_ESSID_MAX_SIZE + 1];
-	if ((event->u.essid.pointer) && (event->u.essid.length))
-	  memcpy (essid, event->u.essid.pointer, event->u.essid.length);
-	essid[event->u.essid.length] = '\0';
-	if (event->u.essid.flags) {
-	  result = essid;
-	  }
-      break;
-
-    case SIOCGIWMODE:
-      if (event->u.mode == 1) { result = i18n("Ad-Hoc"); }
-      else if (event->u.mode == 3) { result = i18n("Managed"); }
-      else result = i18n("Unknown");
-      break;
-
-    case IWEVQUAL:
-      {
-	char buffer[128];
-	WIFI_PRINT_STATS(buffer, event);
-        result = buffer;
-        break;
-      }
-    case SIOCGIWENCODE:
-	if (event->u.data.flags & IW_ENCODE_DISABLED) {
-	  result = i18n( "off" );
-	} else {
-	  result = i18n( "on" );
-	}
-	break;
-   default:
-      break;
-
-  }                           /* switch(event->cmd) */
-
-  return result;
-}
-
-
 QTable* Interface_wireless_wirelessextensions::get_available_networks ()
 {
 
-    QTable* networks = new QTable(0,4,0);
+    networks = new QTable(0,4,0);
     networks->horizontalHeader()->setLabel( 0, i18n("Network Name") );
     networks->horizontalHeader()->setLabel( 1, i18n("Mode") );
     networks->horizontalHeader()->setLabel( 2, i18n("Quality") );
     networks->horizontalHeader()->setLabel( 3, i18n("WEP") );
 
-    /* add 1, then: number of networks found */
-    int netcount = -1;
+    KProcIO *iwlist = new KProcIO;
+    QString iwlist_bin = KStandardDirs::findExe("iwlist");
+    if(iwlist_bin.isEmpty())
+      iwlist_bin = KStandardDirs::findExe("iwlist", "/usr/local/sbin:/usr/sbin:/sbin");
+    if(iwlist_bin.isEmpty())
+      iwlist_bin = "iwlist"; // try our best ;/
 
-    struct iwreq wrq;
-    unsigned char buffer[IW_SCAN_MAX_DATA];	/* Results */
-    struct timeval tv;				/* Select timeout */
-    int	timeout = 5000000;			/* 5s */
+    *iwlist << iwlist_bin << interface_name << "scanning";
 
-    if (interface_name.isEmpty())
-	return networks;
+    connect ( iwlist, SIGNAL ( readReady ( KProcIO * ) ), this, SLOT ( parseScanData ( KProcIO * ) ) );
+	    
+   if ( !iwlist->start ( KProcess::Block ) )
+        KMessageBox::sorry ( 0, i18n ( "Unable to perform the scan. Please make sure the executable \"iwlist\" is in your $PATH." ),
+				i18n ( "Scanning not possible" ) );
 
-    /* Init timeout value -> 250ms */
-    tv.tv_sec = 0;
-    tv.tv_usec = 250000;
-
-    /*
-     * Here we should look at the command line args and set the IW_SCAN_ flags
-     * properly
-     */
-    wrq.u.param.flags = IW_SCAN_DEFAULT;
-    wrq.u.param.value = 0;	/* Later */
-
-    /* Initiate Scanning */
-    wrq.u.data.pointer = NULL;
-    wrq.u.data.flags = 0;
-    wrq.u.data.length = 0;
-    if (iw_set_ext
-	(socket, (char *) interface_name.latin1 (), SIOCSIWSCAN, &wrq) < 0)
-      {
-	if (errno != EPERM)
-	  {
-	    kdWarning() << "Interface does not support scanning (errno=" << errno << ")" << endl;
-	    return networks;
-	  }
-	/* If we don't have the permission to initiate the scan, we may
-	 * still have permission to read left-over results.
-	 * But, don't wait !!! */
-	tv.tv_usec = 0;
-      }
-    timeout -= tv.tv_usec;
-
-    /* Forever */
-    while (1)
-      {
-	fd_set rfds;		/* File descriptors for select */
-	int last_fd;		/* Last fd */
-	int ret;
-
-	/* Guess what ? We must re-generate rfds each time */
-	FD_ZERO (&rfds);
-	last_fd = -1;
-
-	/* In here, add the rtnetlink fd in the list */
-
-	/* Wait until something happens */
-	// ret = select (last_fd + 1, &rfds, NULL, NULL, &tv);
-	ret = sleep(3);
-
-	/* Check if there was an error */
-	if (ret > 0)
-	  {
-	    if (errno == EAGAIN || errno == EINTR)
-	      continue;
-	    kdWarning() << "Unhandled signal - aborting scan..." << endl;
-	    return networks;
-	  }
-
-	/* Check if there was a timeout */
-	if (ret == 0)
-	  {
-	    /* Try to read the results */
-	    wrq.u.data.pointer = (char *) buffer;
-	    wrq.u.data.flags = 0;
-	    wrq.u.data.length = sizeof (buffer);
-	    if (iw_get_ext
-		(socket, (char *) interface_name.latin1 (), SIOCGIWSCAN,
-		 &wrq) < 0)
-	      {
-		/* Check if results not available yet */
-		// if (errno == EAGAIN)
-		  // {
-		    /* Restart timer for only 100ms */
-		    //tv.tv_sec = 0;
-		    //tv.tv_usec = 100000;
-		    //timeout -= tv.tv_usec;
-		    //if (timeout > 0)
-		    //  continue;	/* Try again later */
-		  //}
-
-		/* Bad error */
-		kdWarning() << "Failed to read scan data - aborting scan..." << endl;
-		return networks;
-	      }
-	    else
-	      /* We have the results, go to process them */
-	      break;
-	  }
-
-	/* In here, check if event and event type
-	 * if scan event, read results. All errors bad & no reset timeout */
-      }
-
-    if (wrq.u.data.length)
-      {
-	struct iw_event
-	  iwe;
-	struct stream_descr
-	  stream;
-	int
-	  ret;
-
-	iw_init_event_stream (&stream, (char *) buffer, wrq.u.data.length);
-	do
-	  {
-	    /* Extract an event and print it */
-
-	    ret = WIFI_EXTRACT_EVENT_STREAM(&stream, &iwe);
-
-	    /* Currently, we only return the ESSIDs. We could
-	       collect further data, but that's something for
-	       maybe KDE 3.5 */
-
-	    /* The first token is about ESSID. So if we see that, create a
-	       new row. If it's something else, it belongs to the network
-	       before. */
-
-	    if (ret > 0) {
-	      if (iwe.cmd == SIOCGIWESSID) {
-		 netcount++;
-		 kdDebug() << netcount << "\n";
-		 networks->insertRows(netcount);
-		 QString tempnet = print_scanning_token (&iwe);
-		 if (!tempnet.isEmpty()) 
-			{ networks->setText( netcount, 0, tempnet ); }
-			else 
-			{ networks->setText( netcount, 0, (QString)i18n("(hidden cell)")); }
-	      }
-	      if (iwe.cmd == SIOCGIWMODE) {
-                 QString tempresult= print_scanning_token (&iwe);
-                 networks->setText( netcount, 1, tempresult );
-		 
-	      }
-              if (iwe.cmd == IWEVQUAL) {
-                 QString tempresult= print_scanning_token (&iwe);
-		 // crop the result... it's a little too verbose
-		 // find everything between the first : and the first ' '
-		 tempresult = tempresult.mid(tempresult.find(':') + 1, tempresult.find( ' ', tempresult.find(':') ) - tempresult.find(':') );
-                 networks->setText( netcount, 2, tempresult );
-	      }
-	      if (iwe.cmd == SIOCGIWENCODE) {
-		   QString tempresult = print_scanning_token (&iwe);
-		   networks->setText( netcount, 3, tempresult ); 
-	      }
-	    }
-
-	  }
-	while (ret > 0);
-      }
+    // while (iwlist->isRunning()) sleep ( 1 );
 
     for (int i = 0; i<4; i++ ) networks->adjustColumn(i);
 
     return networks;
 }
 
+void
+Interface_wireless_wirelessextensions::parseScanData ( KProcIO * iwlist )
+{
+  QString data;
+  int cellcount = 0, iteratecount = 0;
+  bool ignoreRemainingBits = false;
+  while ( iwlist->readln ( data, true ) >= 0 )
+    {
+      kdDebug (  ) << "iwlist: " << data << "\n";
+      if ( data.contains ( "does not support scanning" ) )
+        KMessageBox::sorry ( 0,
+                             i18n
+                             ( "Your card does not support scanning. The results window will not contain any results." ),
+                             i18n ( "Scanning not possible" ) );
+      if ( data.contains ( "Scan completed" ) )
+        cellcount = 0;          // at the very beginning of a scan
+      if ( data.contains ( "Cell" ) ) {
+        cellcount++;            // new cell discovered
+	networks->setNumRows ( networks->numRows() +1 );
+	ignoreRemainingBits = false;
+      }
+      if ( data.contains ( "ESSID:" ) )
+        {
+          QString ssid = data.mid ( data.find ( "\"" ) + 1, data.length (  ) - data.find ( "\"" ) - 2 );
+	  if ((ssid=="") || (ssid==" ")) ssid = "(hidden cell)";
+          networks->setText ( cellcount - 1, 0, ssid );
+        }
+      if ( data.contains ( "Mode:" ) )
+        {
+          if ( data.contains ( "Master" ) )
+            networks->setText ( cellcount - 1, 1, QString ( i18n ( "Managed" ) ) );
+          if ( data.contains ( "Ad-Hoc" ) )
+            networks->setText ( cellcount - 1, 1, QString ( i18n ( "Ad-Hoc" ) ) );
+	  // if could be that this cell belongs to an SSID already discovered, or that there are more than one
+	  // hidden cells, which doesn't give any new information. So, we first search for duplicates and delete 
+	  // this row if it's a duplicate. If the same SSID is there once as Managed and once as Ad-Hoc it is no
+	  // duplicate of course, but "something completely different"
+	  for (iteratecount = 0; iteratecount < cellcount - 1; iteratecount++)
+	  {
+	      kdDebug() << "Comparing <" << networks->text(cellcount - 1, 0) << "," << networks->text(iteratecount,0) << " and <"
+		        << networks->text(cellcount - 1, 1) << "," << networks->text(iteratecount,1) << ">.\n";
+	      if ((networks->text(cellcount - 1 , 0) == networks->text(iteratecount,0)) &&
+	          (networks->text(cellcount - 1 , 1) == networks->text(iteratecount,1)) )
+	      {
+	           networks->setNumRows ( networks->numRows() - 1 );
+	           cellcount--;
+		   ignoreRemainingBits = true;
+	      }
+	  }
+        }
+      if ( !ignoreRemainingBits && data.contains ( "Encryption key:" ) )
+        {
+          if ( data.contains ( "on" ) )
+            networks->setText ( cellcount - 1, 3, QString ( "on" ) );
+          else
+            networks->setText ( cellcount - 1, 3, QString ( "off" ) );
+        }
+      if ( !ignoreRemainingBits && data.contains ( "Quality:" ) )
+        {
+          QString quality = data.mid ( data.find ( ":" ) + 1, data.find ( "/" ) - data.find ( ":" ) - 1 );
+          networks->setText ( cellcount - 1, 2, quality );
+        }
+
+    }
+}
+
 #include "interface_wireless_wirelessextensions.moc"
